<?php

namespace ACL\Controllers;

use ACL\Traits\TResponse;
use Exception;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use ACL\Helpers\Instances\InstanceProvider;
use ACL\Util\AclUtilities;
use ACL\Services\Organizacao as OrganizacaoService;

class Organizacao {

    use TResponse;

    /**
     * @var OrganizacaoService
     */
    private $organizacaoService;

    public function __construct(OrganizacaoService $organizacaoService)
    {
        $this->organizacaoService = $organizacaoService;
    }

    /**
     * @param Request $request
     * @param Response $response
     * @throws Exception
     */
    public function findOrganizacao(Request $request, Response $response)
    {
        try {
            $organization = $this->organizacaoService->findOrganizacao();
            $this->sendResponse($organization, 200);
        } catch (Exception $e) {
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }

    /**
     * @param Request $request
     * @throws Exception
     */
    public function findOrgOrgoper(Request $request)
    {
        try {
            $params = $request->getParameters();
            $cdoperador = $request->getParameter('CDOPERADOR');
            $toRelate = !empty($params['TO_RELATE']) ? $params['TO_RELATE'] : null ;
            $organizations = $this->organizacaoService->findOrgOrgoper($cdoperador, $toRelate);
            $this->sendResponse($organizations, 200);
        } catch (Exception $e) {
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }

    /**
     * @param Request $request
     * @throws Exception
     */
    public function createOrgOrgoper(Request $request)
    {
        try {
            $params = $request->getParameters();
            $this->organizacaoService->createOrgOrgoper($params);
            $this->sendResponse([], 201);
        } catch (Exception $e) {
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }

    /**
     * @param Request $request
     * @throws \Doctrine\DBAL\Driver\Exception
     */
    public function deleteOrganizacaooper(Request $request)
    {
        try {
            $params = $request->getParameters();
            $this->organizacaoService->deleteOrganizacaooper($params);
            $this->sendResponse(null, 204);
        } catch (Exception $e) {
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }
}
