<?php

namespace ACL\Repositories;

use ACL\Traits\TQuery;
use ACL\Helpers\Instances\InstanceProvider;
use Doctrine\DBAL\Exception;
use Doctrine\ORM\EntityManager;

class Filial extends RepositoryBase {

    Use TQuery;

    protected $table = 'FILIAL';

    /** @var EntityManager */
    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        parent::__construct($this->table);
        $this->entityManager = $entityManager;
    }

    /**
     * @param $cdoperador
     * @param $nrorg
     * @param bool $isFilterNotInFilial
     * @return array
     */
    public function findFilial($nrorg, $isFilterNotInFilial = null, $cdoperador = null): array
    {
        $filterNotInFilial = $isFilterNotInFilial
                             ? "AND FILI.CDFILIAL NOT IN (SELECT CDFILIAL FROM FILIOPER WHERE :CDOPERADOR = CDOPERADOR)"
                             : "";
        $query = <<<SQL
            SELECT *
              FROM (SELECT DISTINCT FILI.*, PA.NMPAIS, ES.NMESTADO, MU.NMMUNICIPIO, RE.NMREGIAO 
                      FROM FILIAL FILI 
                      JOIN ENDEFILI EM ON FILI.CDFILIAL = EM.CDFILIAL
                                      AND FILI.NRORG = EM.NRORG
                                      AND EM.IDTPENDEFILI = 'P'
                 LEFT JOIN PAIS PA ON EM.CDPAIS = PA.CDPAIS
                                  AND EM.NRORG = PA.NRORG
                 LEFT JOIN ESTADO ES ON EM.SGESTADO = ES.SGESTADO
                                    AND EM.CDPAIS = ES.CDPAIS
                                    AND EM.NRORG = ES.NRORG
                 LEFT JOIN MUNICIPIO MU ON EM.CDMUNICIPIO = MU.CDMUNICIPIO
                                       AND EM.SGESTADO = MU.SGESTADO
                                       AND EM.CDPAIS = MU.CDPAIS
                                       AND EM.NRORG = MU.NRORG
                 LEFT JOIN REGIAO RE ON FILI.CDREGIAO = RE.CDREGIAO
                                    AND EM.SGESTADO = RE.SGESTADO
                                    AND EM.CDPAIS = RE.CDPAIS
                                    AND EM.NRORG = RE.NRORG
                     WHERE 1 = 1
                       $filterNotInFilial
                       AND FILI.NRORG = :NRORG)
             WHERE 1 = 1
          ORDER BY CDFILIAL
SQL;
        $params = array(
            'CDOPERADOR' => $cdoperador,
            'NRORG' => $nrorg
        );
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $query, $params);
    }
}
