/* Sistemas Integrados TEKNISA
  Integrao

Nome: 0100_MAKSTPO.SQL

Descrio:
Gera de vendas vindas do sistema PDV - Makro.

@C:\web_550\sql\end_conv\oracle\clientes\mak\0100_MAKSTPO_IMPORTA_VENDA_REDZ.sql

P_CDFILIAL - Filial
P_DATA     - Data
P_ERRO     - Retorna erros encontrados.

Criao:
13/08/2012 - Fabrcio Marcelo da Silva
*/

CREATE OR REPLACE PROCEDURE USR_GERA_VENDA
   (P_CDFILIAL    IN FILIAL.CDFILIAL%TYPE,
    P_DATA        IN DATE,
    P_ERRO        OUT VARCHAR2)
AS
   V_DTABERTURA      DATE;
   V_DTFECHAMEN      DATE;
   V_NRSEQUITVEND    VARCHAR2(03);

   V_CDLOJAEXIST     VARCHAR2(10);
   V_CDCAIXAEXIST    VARCHAR2(03);

   V_TOTALITENS      NUMBER(12,2);
   V_TOTALDESCONTO   NUMBER(12,2);
   V_TURCAIXA        VARCHAR2(04);
   V_CODVENDA        VARCHAR2(20);

   V_CDOPERADOR      VARCHAR2(12);
   V_CDCLIENTE       VARCHAR2(21);
   V_CDEMPRESA       VARCHAR2(02);

   V_CONTPRODUTO     INTEGER;
   V_CONTPRODCAD     INTEGER;

   V_DTREDUCAOZ      DATE;

   V_ENTROU          VARCHAR2(01);
   V_JEXISREDZ       VARCHAR2(04);
   V_ERROVENDA       VARCHAR2(01);
BEGIN
   V_ENTROU := 'N';

   P_ERRO  := NULL;

   FOR CUR_VENDAS IN (SELECT V.CDFILIAL AS CDFILIPDV, P.CDFILIAL, V.CDLOJA, V.CDCAIXA, V.NRSEQVENDA,
                             V.DTVENDA, NVL(V.VRTOTALVENDA,0) AS VRTOTALVENDA, NVL(V.VRTOTALDESCONTO,0) AS VRTOTALDESCONTO
                        FROM USR_CABEC_VENDA V, PARAMFILIAL P
                       WHERE P.CDFILIAL = P_CDFILIAL
                         AND P.CDEXFILIAL = V.CDFILIAL
                         AND TO_DATE(V.DTVENDA) = TO_DATE(P_DATA)
                         AND V.DTIMPORTACAO IS NULL
                         AND EXISTS (SELECT I.CDFILIAL
                                       FROM USR_ITENS_VENDA I
                                      WHERE I.CDFILIAL = V.CDFILIAL
                                        AND I.CDCAIXA = V.CDCAIXA
                                        AND I.NRSEQVENDA = V.NRSEQVENDA)
                       ORDER BY V.CDFILIAL, V.CDLOJA, V.CDCAIXA, V.DTVENDA, V.NRSEQVENDA)
   LOOP
      V_ENTROU := 'S';
      V_ERROVENDA := 'N';
      
      /* Verifica se a loja esta cadastrada no sistema VND */
      BEGIN
         SELECT CDLOJA
           INTO V_CDLOJAEXIST
           FROM LOJA
          WHERE CDFILIAL = CUR_VENDAS.CDFILIAL
            AND CDLOJA = CUR_VENDAS.CDLOJA;
      EXCEPTION
         WHEN NO_DATA_FOUND THEN
            V_CDLOJAEXIST := NULL;
      END;

      IF (NOT V_CDLOJAEXIST IS NULL) THEN

         /* Verifica se o caixa esta cadastrada no sistema VND */
         BEGIN
            SELECT CDCAIXA
              INTO V_CDCAIXAEXIST
              FROM CAIXA
             WHERE CDFILIAL = CUR_VENDAS.CDFILIAL
               AND CDCAIXA = CUR_VENDAS.CDCAIXA;
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_CDCAIXAEXIST := NULL;
         END;

         IF (NOT V_CDCAIXAEXIST IS NULL) THEN

            V_TOTALITENS    := 0;
            V_TOTALDESCONTO := 0;

            SELECT NVL(SUM(VRQUANTIDADE * VRVALRUNIT),0)
              INTO V_TOTALITENS
              FROM USR_ITENS_VENDA
             WHERE CDFILIAL = CUR_VENDAS.CDFILIPDV
               AND CDCAIXA = CUR_VENDAS.CDCAIXA
               AND NRSEQVENDA = CUR_VENDAS.NRSEQVENDA;

            SELECT SUM(NVL(VRDESCONTO,0))
              INTO V_TOTALDESCONTO
              FROM USR_ITENS_VENDA
             WHERE CDFILIAL = CUR_VENDAS.CDFILIPDV
               AND CDCAIXA = CUR_VENDAS.CDCAIXA
               AND NRSEQVENDA = CUR_VENDAS.NRSEQVENDA;


            IF (ROUND(V_TOTALITENS,2) = ROUND(CUR_VENDAS.VRTOTALVENDA,2)) THEN
               IF (ROUND(V_TOTALDESCONTO,2) = ROUND(CUR_VENDAS.VRTOTALDESCONTO,2)) THEN

                  V_DTABERTURA := TO_DATE(TO_CHAR(TRUNC(CUR_VENDAS.DTVENDA), 'DD/MM/YYYY') || '0600', 'DD/MM/YYYY HH24MI');
                  V_DTFECHAMEN := TO_DATE(TO_CHAR(TRUNC(CUR_VENDAS.DTVENDA), 'DD/MM/YYYY') || '2300', 'DD/MM/YYYY HH24MI');

                  /* Verifica operador, cliente e empresa padro utilizados para abertura e fechamento do caixa */
                  BEGIN
                     SELECT P.CDOPERADOR, P.CDCLIENTE, F.CDEMPRESA
                       INTO V_CDOPERADOR, V_CDCLIENTE, V_CDEMPRESA
                       FROM PARAVEND P, FILIAL F
                      WHERE P.CDFILIAL = CUR_VENDAS.CDFILIAL
                        AND P.CDPARAVEND = '1'
                        AND P.CDFILIAL = F.CDFILIAL;
                  EXCEPTION
                     WHEN NO_DATA_FOUND THEN
                        V_CDOPERADOR := NULL;
                        V_CDCLIENTE  := NULL;
                        V_CDEMPRESA  := NULL;
                  END;

                  IF ((NOT V_CDOPERADOR IS NULL)  AND
                      (NOT V_CDCLIENTE IS NULL)   AND
                      (NOT V_CDEMPRESA IS NULL))  THEN

                     /* Verifica se o caixa j foi aberto */
                     BEGIN
                        SELECT CDFILIAL
                          INTO V_TURCAIXA
                          FROM TURCAIXA
                         WHERE CDFILIAL = CUR_VENDAS.CDFILIAL
                           AND CDCAIXA = CUR_VENDAS.CDCAIXA
                           AND DTABERCAIX = V_DTABERTURA;
                     EXCEPTION
                        WHEN NO_DATA_FOUND THEN
                           V_TURCAIXA := NULL;
                     END;

                     IF (V_TURCAIXA IS NULL) THEN
                        /* Efetue a abertura do caixa */
                        INSERT INTO TURCAIXA
                           (CDFILIAL, CDCAIXA, DTABERCAIX, CDOPERABER, DTMOVTURCAIX,
                            CDOPERADOR, DTFECHCAIX, CDOPERFECH)
                        VALUES
                           (CUR_VENDAS.CDFILIAL, CUR_VENDAS.CDCAIXA, V_DTABERTURA, V_CDOPERADOR, TRUNC(CUR_VENDAS.DTVENDA),
                            V_CDOPERADOR, V_DTFECHAMEN, V_CDOPERADOR);
                     END IF;

                     NOVO_CODIGO('PEDIDOVENDA'||CUR_VENDAS.CDFILIAL||CUR_VENDAS.CDCAIXA, V_CODVENDA);
                     V_CODVENDA := SUBSTR(V_CODVENDA, 11, 10);

                     /* Efetue a incluso das vendas */
                     INSERT INTO VENDA
                        (CDFILIAL, CDCAIXA, NRSEQVENDA, CDCLIENTE, CDEMPRESA,
                         CDCONSUMIDOR, DTVENDA, DTENTRVENDA, IDSITUVENDA, CDFILIALTUR,
                         CDCAIXATUR, DTABERTUR, CDSERVFILI, CDOPERADOR, CDLOJA,
                         IDTIPOVENDA, CDCCUSCLIE)
                     VALUES
                        (CUR_VENDAS.CDFILIAL, CUR_VENDAS.CDCAIXA, V_CODVENDA, V_CDCLIENTE, V_CDEMPRESA,
                         NULL, CUR_VENDAS.DTVENDA, TO_DATE(CUR_VENDAS.DTVENDA, 'DD/MM/YYYY'), 'O', CUR_VENDAS.CDFILIAL,
                         CUR_VENDAS.CDCAIXA, V_DTABERTURA, NULL, V_CDOPERADOR, CUR_VENDAS.CDLOJA,
                         'I', NULL);

                     V_NRSEQUITVEND := '001';

                     /* Verifica quantos produtos foram vendidos por venda */
                     BEGIN
                        SELECT COUNT(CDPRODUTO)
                          INTO V_CONTPRODUTO
                          FROM USR_ITENS_VENDA
                         WHERE CDFILIAL = CUR_VENDAS.CDFILIPDV
                           AND CDCAIXA = CUR_VENDAS.CDCAIXA
                           AND NRSEQVENDA = CUR_VENDAS.NRSEQVENDA;
                     EXCEPTION
                        WHEN NO_DATA_FOUND THEN
                           V_CONTPRODUTO := 0;
                     END;

                     /* Verifica se os produtos vendidos estao cadastrados no sistema VND */
                     BEGIN
                        SELECT COUNT(I.CDPRODUTO)
                          INTO V_CONTPRODCAD
                          FROM USR_ITENS_VENDA I, PRODUTO P
                         WHERE I.CDFILIAL = CUR_VENDAS.CDFILIPDV
                           AND I.CDCAIXA = CUR_VENDAS.CDCAIXA
                           AND I.NRSEQVENDA = CUR_VENDAS.NRSEQVENDA
                           AND P.CDARVPROD = I.CDPRODUTO
                           AND P.CDNVPRODUTO = '5';
                     EXCEPTION
                        WHEN NO_DATA_FOUND THEN
                           V_CONTPRODCAD := 0;
                     END;

                     IF (V_CONTPRODUTO <> 0) THEN

                        IF (V_CONTPRODUTO = V_CONTPRODCAD) THEN

                           FOR CUR_ITENS IN (SELECT P.CDPRODUTO, I.VRQUANTIDADE, I.VRVALRUNIT, I.VRDESCONTO
                                               FROM USR_ITENS_VENDA I, PRODUTO P
                                              WHERE I.CDFILIAL = CUR_VENDAS.CDFILIPDV
                                                AND I.CDCAIXA = CUR_VENDAS.CDCAIXA
                                                AND I.NRSEQVENDA = CUR_VENDAS.NRSEQVENDA
                                                AND P.CDARVPROD = I.CDPRODUTO
                                                AND P.CDNVPRODUTO = '5')
                           LOOP
                              INSERT INTO ITEMVENDA
                                 (CDFILIAL, CDCAIXA, NRSEQVENDA, NRSEQUITVEND,
                                  QTPRODVEND, VRUNITVEND, VRUNITVENDCL, VRACRITVEND,
                                  VRDESITVEND, CDPRODUTO)
                              VALUES
                                 (CUR_VENDAS.CDFILIAL, CUR_VENDAS.CDCAIXA, V_CODVENDA, V_NRSEQUITVEND,
                                  CUR_ITENS.VRQUANTIDADE, CUR_ITENS.VRVALRUNIT, 0, 0,
                                  CUR_ITENS.VRDESCONTO, CUR_ITENS.CDPRODUTO);

                              V_NRSEQUITVEND := LPAD(RTRIM(TO_CHAR(TO_NUMBER(V_NRSEQUITVEND) + 1)),3,'0');
                           END LOOP;
                        ELSE
                           DELETE FROM VENDA
                            WHERE CDFILIAL = CUR_VENDAS.CDFILIAL
                              AND CDCAIXA = CUR_VENDAS.CDCAIXA
                              AND NRSEQVENDA = V_CODVENDA;

                           /* Grava o erro encontrado na importao da venda */
                           UPDATE USR_CABEC_VENDA
                              SET DSOBSERVERROS = 'Possui produtos vendidos no cadastrados no sistema Teknisa.'
                            WHERE CDFILIAL = CUR_VENDAS.CDFILIPDV
                              AND CDCAIXA = CUR_VENDAS.CDCAIXA
                              AND NRSEQVENDA = CUR_VENDAS.NRSEQVENDA;

                           P_ERRO := 'V';
                           V_ERROVENDA := 'S';
                        END IF;
                     ELSE
                        DELETE FROM VENDA
                         WHERE CDFILIAL = CUR_VENDAS.CDFILIAL
                           AND CDCAIXA = CUR_VENDAS.CDCAIXA
                           AND NRSEQVENDA = V_CODVENDA;

                        /* Grava o erro encontrado na importao da venda */
                        UPDATE USR_CABEC_VENDA
                           SET DSOBSERVERROS = 'Venda no possui produtos.'
                         WHERE CDFILIAL = CUR_VENDAS.CDFILIPDV
                           AND CDCAIXA = CUR_VENDAS.CDCAIXA
                           AND NRSEQVENDA = CUR_VENDAS.NRSEQVENDA;

                        P_ERRO := 'V';
                        V_ERROVENDA := 'S';
                     END IF;

                     IF (V_ERROVENDA = 'N') THEN
                        /* Atualiza a data de importao da venda */
                        UPDATE USR_CABEC_VENDA
                           SET DTIMPORTACAO = SYSDATE,
                               DSOBSERVERROS = NULL
                         WHERE CDFILIAL = CUR_VENDAS.CDFILIPDV
                           AND CDCAIXA = CUR_VENDAS.CDCAIXA
                           AND NRSEQVENDA = CUR_VENDAS.NRSEQVENDA;
                     END IF;
                  ELSE
                     P_ERRO := 'Operador, cliente ou empresa no relacionado para gerao de vendas.';
                  END IF;
               ELSE
                  /* Grava o erro encontrado na importao da venda */
                  UPDATE USR_CABEC_VENDA
                     SET DSOBSERVERROS = 'O vlr. total de descontos dos produtos difere do vlr. total de descontos da venda.'
                   WHERE CDFILIAL = CUR_VENDAS.CDFILIPDV
                     AND CDCAIXA = CUR_VENDAS.CDCAIXA
                     AND NRSEQVENDA = CUR_VENDAS.NRSEQVENDA;

                  P_ERRO := 'V';
                  V_ERROVENDA := 'S';
               END IF;
            ELSE
               /* Grava o erro encontrado na importao da venda */
               UPDATE USR_CABEC_VENDA
                  SET DSOBSERVERROS = 'O vlr. total dos produtos difere do vlr. total da venda.'
                WHERE CDFILIAL = CUR_VENDAS.CDFILIPDV
                  AND CDCAIXA = CUR_VENDAS.CDCAIXA
                  AND NRSEQVENDA = CUR_VENDAS.NRSEQVENDA;

               P_ERRO := 'V';
               V_ERROVENDA := 'S';
            END IF;
         ELSE
            /* Grava o erro encontrado na importao da venda */
            UPDATE USR_CABEC_VENDA
               SET DSOBSERVERROS = 'Caixa ' || CUR_VENDAS.CDCAIXA || ' no cadastrado no sistema Teknisa.'
             WHERE CDFILIAL = CUR_VENDAS.CDFILIPDV
               AND CDCAIXA = CUR_VENDAS.CDCAIXA
               AND NRSEQVENDA = CUR_VENDAS.NRSEQVENDA;

            P_ERRO := 'V';
            V_ERROVENDA := 'S';
         END IF;
      ELSE
         /* Grava o erro encontrado na importao da venda */
         UPDATE USR_CABEC_VENDA
            SET DSOBSERVERROS = 'Loja '|| CUR_VENDAS.CDLOJA || ' no cadastrada no sistema Teknisa.'
          WHERE CDFILIAL = CUR_VENDAS.CDFILIPDV
            AND CDCAIXA = CUR_VENDAS.CDCAIXA
            AND NRSEQVENDA = CUR_VENDAS.NRSEQVENDA;

         P_ERRO := 'V';
         V_ERROVENDA := 'S';
      END IF;
      COMMIT;
   END LOOP;

   /* Caso no tenha sido importada nenhuma venda para o caixa  excludo o registro de abertura do caixa */

   FOR CUR_TURCAIXA IN (SELECT T.CDFILIAL, T.CDCAIXA, T.DTABERCAIX
                          FROM TURCAIXA T
                         WHERE T.CDFILIAL = P_CDFILIAL
                           AND TO_DATE(T.DTABERCAIX) = TO_DATE(P_DATA)
                           AND NOT EXISTS (SELECT V.CDFILIAL
                                             FROM VENDA V
                                            WHERE V.CDFILIAL = T.CDFILIAL
                                              AND V.CDCAIXA = T.CDCAIXA
                                              AND V.DTABERTUR = T.DTABERCAIX))
   LOOP
      DELETE FROM TURCAIXA
       WHERE CDFILIAL = CUR_TURCAIXA.CDFILIAL
         AND CDCAIXA = CUR_TURCAIXA.CDCAIXA
         AND DTABERCAIX = CUR_TURCAIXA.DTABERCAIX;
   END LOOP;
   COMMIT;

   /* Incluso da reducao Z caso o caixa tenha sido aberto */

   FOR CUR_REDUCAOZ IN (SELECT R.CDFILIAL AS CDFILIREDZ, P.CDFILIAL, CDCAIXA, NRREDUCAOZ, DTREDUCAOZ,
                               REINICIO, SERIEREDZ, CONTREDZ, LEITURAX,
                               COMPNFISCAL, CANCCUPFISC, CUPFISCINI, CUPFISCFIN,
                               VRTOTALINI, VRTOTALFIN, VRVENDABRUT, VRTOTALISS,
                               VRTOTALCANC, VRTOTALDESC, VRVENDALIQU, VRTOTALACRE,
                               ORIGEMREDU
                          FROM USR_REDUCAOZ R, PARAMFILIAL P
                         WHERE P.CDFILIAL = P_CDFILIAL
                           AND P.CDEXFILIAL = R.CDFILIAL
                           AND R.DTIMPORTACAO IS NULL
                           AND TO_DATE(R.DTREDUCAOZ) = TO_DATE(P_DATA))
   LOOP
      V_ENTROU := 'S';
      IF (CUR_REDUCAOZ.CUPFISCINI <= CUR_REDUCAOZ.CUPFISCFIN) THEN

            V_DTREDUCAOZ := TO_DATE(TO_CHAR(TRUNC(CUR_REDUCAOZ.DTREDUCAOZ), 'DD/MM/YYYY') || '2300', 'DD/MM/YYYY HH24MI');

            /* Verifica se o caixa j foi aberto */
            BEGIN
               SELECT CDFILIAL
                 INTO V_TURCAIXA
                 FROM TURCAIXA
                WHERE CDFILIAL = CUR_REDUCAOZ.CDFILIAL
                  AND CDCAIXA = CUR_REDUCAOZ.CDCAIXA
                  AND TO_DATE(DTABERCAIX) = TO_DATE(V_DTREDUCAOZ);
            EXCEPTION
               WHEN NO_DATA_FOUND THEN
                  V_TURCAIXA := NULL;
            END;

            /* Verifica se a reduo Z j foi importada */
            BEGIN
               SELECT CDFILIAL
                 INTO V_JEXISREDZ
                 FROM REDUCAOZ
                WHERE CDFILIAL = CUR_REDUCAOZ.CDFILIAL
                  AND CDCAIXA = CUR_REDUCAOZ.CDCAIXA
                  AND NRECFREDZ = CUR_REDUCAOZ.NRREDUCAOZ
                  AND DTREDUCAOZ = CUR_REDUCAOZ.DTREDUCAOZ;
            EXCEPTION
               WHEN NO_DATA_FOUND THEN
                  V_JEXISREDZ := NULL;
            END;

            IF (NOT V_TURCAIXA IS NULL)  AND
               (V_JEXISREDZ IS NULL)     THEN
               INSERT INTO REDUCAOZ
                  (CDFILIAL, CDCAIXA, NRECFREDZ, DTREDUCAOZ,
                   NRREINIREDZ, NRSERIEREDZ, NRLEITREDZ, NRGERCNFREDZ,
                   NRCANCFREDZ, NRPRIMCFREDZ, VRGTREDZ, NRREDREDZ,
                   VRVENBRUREDZ, VRISSREDZ, VRTOTCANREDZ, VRTOTDESREDZ,
                   VRVENLIQREDZ, VRTOTACRREDZ, NRULTCFREDZ, IDORIGEMREDZ)
               VALUES
                  (CUR_REDUCAOZ.CDFILIAL, CUR_REDUCAOZ.CDCAIXA, CUR_REDUCAOZ.NRREDUCAOZ, CUR_REDUCAOZ.DTREDUCAOZ,
                   CUR_REDUCAOZ.REINICIO, CUR_REDUCAOZ.SERIEREDZ, CUR_REDUCAOZ.LEITURAX, CUR_REDUCAOZ.COMPNFISCAL,
                   CUR_REDUCAOZ.CANCCUPFISC, CUR_REDUCAOZ.CUPFISCINI, CUR_REDUCAOZ.VRTOTALFIN, CUR_REDUCAOZ.CONTREDZ,
                   CUR_REDUCAOZ.VRVENDABRUT, CUR_REDUCAOZ.VRTOTALISS, CUR_REDUCAOZ.VRTOTALCANC, CUR_REDUCAOZ.VRTOTALDESC,
                   CUR_REDUCAOZ.VRVENDALIQU, CUR_REDUCAOZ.VRTOTALACRE, CUR_REDUCAOZ.CUPFISCFIN, 'IMP');

               UPDATE USR_REDUCAOZ
                  SET DTIMPORTACAO = SYSDATE,
                      DSOBSERVERROS = NULL
                WHERE CDFILIAL = CUR_REDUCAOZ.CDFILIREDZ
                  AND CDCAIXA = CUR_REDUCAOZ.CDCAIXA
                  AND NRREDUCAOZ = CUR_REDUCAOZ.NRREDUCAOZ
                  AND DTREDUCAOZ = CUR_REDUCAOZ.DTREDUCAOZ;
            ELSE
               /* Grava o erro encontrado na importao da Reduo Z */
               UPDATE USR_REDUCAOZ
                  SET DSOBSERVERROS = 'Reduo Z j importada ou no existem vendas para o dia.'
                WHERE CDFILIAL = CUR_REDUCAOZ.CDFILIREDZ
                  AND CDCAIXA = CUR_REDUCAOZ.CDCAIXA
                  AND NRREDUCAOZ = CUR_REDUCAOZ.NRREDUCAOZ
                  AND DTREDUCAOZ = CUR_REDUCAOZ.DTREDUCAOZ;

               P_ERRO := 'R';
            END IF;
      ELSE
         /* Grava o erro encontrado na importao da Reduo Z */
         UPDATE USR_REDUCAOZ
            SET DSOBSERVERROS = 'Cupom Fiscal Inicial deve ser menor ou igual ao Cupom Fiscal Final.'
          WHERE CDFILIAL = CUR_REDUCAOZ.CDFILIREDZ
            AND CDCAIXA = CUR_REDUCAOZ.CDCAIXA
            AND NRREDUCAOZ = CUR_REDUCAOZ.NRREDUCAOZ
            AND DTREDUCAOZ = CUR_REDUCAOZ.DTREDUCAOZ;

         P_ERRO := 'R';
      END IF;
   END LOOP;

   COMMIT;

   IF V_ENTROU = 'N' THEN
      P_ERRO := 'No h vendas e/ou reduo Z a serem importadas.';
   END IF;

   IF P_ERRO = 'V' THEN
     P_ERRO := 'Ocorreram erros durante a importao das vendas.';
   END IF;

   IF P_ERRO = 'R' THEN
     P_ERRO := 'Ocorreram erros durante a importao da Reduo Z.';
   END IF;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('USR_GERA_VENDA','5.50.XXX'); END;
/
