/******************************************************************************/
/* Sistemas Integrados TECNISA                                                */
/*                                                                            */
/* LIB (ALTERAR BANCO DE DADOS)                                               */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Nome: 9702_LIBSTP_INSERTCOLUMNS                                            */
/*                                                                            */
/* Descrio:                                                                 */
/*    Insere as colunas de uma determinada tabela na tabela COLUNA           */
/*                                                                            */
/* Criao:                                                                   */
/*    18/08/2000 - Peterson Batista P. Silva                                  */
/*                                                                            */
/* Manutenes:                                                               */
/*   /  /   -                                                                 */
/*                                                                            */
/******************************************************************************/

IF EXISTS(SELECT name FROM sysobjects
          WHERE name = 'INSERTCOLUMNS' AND type = 'P')
  DROP PROCEDURE INSERTCOLUMNS
GO

CREATE PROCEDURE INSERTCOLUMNS
                     @P_NMTABELA  VARCHAR(30)
AS

   DECLARE @V_PRIMARY INT;
   DECLARE @V_EXIST INT;
   DECLARE @V_TABLE_NAME varchar(30);
   DECLARE @V_COLUMN_NAME varchar(30);
   DECLARE @V_DATA_TYPE varchar(9);
   DECLARE @V_DATA_LENGTH INT;
   DECLARE @V_NULLABLE varchar(1);
   DECLARE @V_IDCRIADO varchar(1);
BEGIN

   DELETE FROM COLUNA
   WHERE
      (NMTABELABD = @P_NMTABELA)  AND
      ((IDCRIADOCOL = 'S')    AND
       (NMTABELABD+NMCOLUNA) NOT IN
       (SELECT
           O.NAME + C.NAME
        FROM
           SYSOBJECTS O, SYSCOLUMNS C, SYSUSERS U
        WHERE
           U.UID = O.UID AND
           U.NAME = USER AND
           C.ID = O.ID AND
           O.NAME = @P_NMTABELA));

   DECLARE C_COLUNA CURSOR FOR
      SELECT
         O.NAME       AS TABLE_NAME,
         C.NAME       AS COLUMN_NAME,
         T.NAME       AS DATA_TYPE,
         C.LENGTH     AS DATA_LENGTH,
         C.ISNULLABLE AS NULLABLE,
         'S'          AS IDCRIADO
      FROM
         SYSCOLUMNS C, SYSOBJECTS O, SYSTYPES T, SYSUSERS U
      WHERE
         (C.ID = O.ID)       AND
         (O.XTYPE = 'U')     AND
         (C.XTYPE = T.XTYPE) AND
         (U.NAME  = USER)    AND
         (O.UID = U.UID)     AND
         (UPPER(O.NAME) = @P_NMTABELA)

      OPEN C_COLUNA;
      FETCH NEXT FROM C_COLUNA INTO @V_TABLE_NAME,
                                    @V_COLUMN_NAME,
                                    @V_DATA_TYPE,
                                    @V_DATA_LENGTH,
                                    @V_NULLABLE,
                                    @V_IDCRIADO;

   WHILE (@@FETCH_STATUS = 0)
   BEGIN

       IF @V_NULLABLE = '0' BEGIN
          SET @V_NULLABLE = 'N'
       END
       ELSE BEGIN
          SET @V_NULLABLE = 'Y'
       END;

       UPDATE COLUNA
       SET
          NRTAMANHOCOL = @V_DATA_LENGTH,
          IDCOLUNANULL = @V_NULLABLE
       WHERE
          (NMTABELABD = @P_NMTABELA)            AND
          (NMCOLUNA   = @V_COLUMN_NAME);

       SELECT
          @V_PRIMARY = COUNT(*)
       FROM
          SYSOBJECTS O, SYSINDEXES I, SYSUSERS U,
          SYSINDEXKEYS IK, SYSCOLUMNS C
       WHERE
          O.UID = U.UID        AND
          U.NAME = USER        AND
          O.ID = I.ID          AND
          O.XTYPE = 'U'        AND
          IK.ID = O.ID         AND
          IK.INDID = I.INDID   AND
          O.ID = C.ID          AND
          IK.COLID = C.COLID   AND
          I.NAME LIKE 'PK%'    AND
          O.NAME = @P_NMTABELA AND
          C.NAME = @V_COLUMN_NAME ;

       SELECT @V_EXIST = COUNT(*)
       FROM
          COLUNA
       WHERE
          NMTABELABD = @V_TABLE_NAME AND
          NMCOLUNA   = @V_COLUMN_NAME;

       IF UPPER(@V_DATA_TYPE) = 'VARCHAR' BEGIN
          SET @V_DATA_TYPE = 'C'
       END
       ELSE IF UPPER(@V_DATA_TYPE) = 'DATETIME' BEGIN
          SET @V_DATA_TYPE = 'D'
       END ELSE IF UPPER(@V_DATA_TYPE) = 'NUMERIC' BEGIN
          SET @V_DATA_TYPE = 'N'
       END ELSE BEGIN
          SET @V_DATA_TYPE = 'X';
       END;

       IF @V_EXIST = 0 BEGIN
          IF @V_PRIMARY = 0 BEGIN
             INSERT INTO COLUNA (NMTABELABD            , NMCOLUNA,
                                  IDTIPOCOLUNA          , NRTAMANHOCOL,
                                  IDCOLUNANULL          , IDCRIADOCOL,
                                  IDPRIMARY)
             VALUES              (@V_TABLE_NAME, @V_COLUMN_NAME,
                                  @V_DATA_TYPE , @V_DATA_LENGTH,
                                  @V_NULLABLE  , @V_IDCRIADO,
                                  NULL);
          END
          ELSE BEGIN
             INSERT INTO COLUNA (NMTABELABD            , NMCOLUNA,
                                  IDTIPOCOLUNA          , NRTAMANHOCOL,
                                  IDCOLUNANULL          , IDCRIADOCOL,
                                  IDPRIMARY)
             VALUES              (@V_TABLE_NAME, @V_COLUMN_NAME,
                                  @V_DATA_TYPE , @V_DATA_LENGTH,
                                  @V_NULLABLE  , @V_IDCRIADO,
                                  'S');
          END;
       END;
       PRINT @V_COLUMN_NAME;
       FETCH NEXT FROM C_COLUNA INTO @V_TABLE_NAME,
                                     @V_COLUMN_NAME,
                                     @V_DATA_TYPE,
                                     @V_DATA_LENGTH,
                                     @V_NULLABLE,
                                     @V_IDCRIADO
   END;
   CLOSE C_COLUNA;
   DEALLOCATE C_COLUNA;
END;

GO

