import { Http, Metadata, Singleton } from '@zeedhi/core';
import { TekGrid, TekRestDatasource } from '@zeedhi/teknisa-components-common';
import { Select, Tabs } from '@zeedhi/common';
import { OperadorService } from '@/services/OperadorService';
import { PmLibDialogService, PmLibFormSave, PmLibUtil } from '@zeedhi/pm-lib';
import { Utilities } from '@/Util/Utilities';

@Singleton
export class OperadorController {
	public operadorService!: OperadorService;

	public isVisibleGridAndDetail = true;

	public nrorg = '';

	public nmorganizacao = '';

	public operador: any = {};

	public showFullOperatorWindow = false;

	public filialRuleInFullOperatorWindow = false;

	private setOperadorInfo(data: any) {
		this.nmorganizacao = data.NMORG;
		this.nrorg = data.NRORG;
		this.operador.ACL_ADMINISTRATOR = data.ACL_ADMINISTRATOR;
		this.operador.CDOPERADOR = data.CDOPERADOR;
		this.operador.NMOPERADOR = data.NMOPERADOR;
		this.operador.NRORG = data.NRORG;
		this.operador.NMORG = data.NMORG;
		this.operador.CDFILIAL = data.CDFILIAL;
		this.operador.IDOPERMASTER = data.IDOPERMASTER;
		this.operador.IDOPERMASTERLOGGED = data.IDOPERMASTER;
		this.operador.NRORGTRAB = data.NRORGTRAB;
		this.operador.NMORGTRAB = data.NMORGTRAB;
		this.operador.NRORGTRABLOGGED = data.NRORGTRAB;
		this.operador.NRESTRUTURAM = data.NRESTRUTURAM;
		this.operador.NMESTRUTURAM = data.NMESTRUTURAM;
	}

	public promiseOperadorInfo: any = null;

	public promiseOrgoper: any = null;

	public currentOperador: any = {};

	public backupRow: any = {};

	public isReloadOperadorGrid = false;

	constructor() {
		this.operadorService = new OperadorService();
		this.showFullOperatorWindow = JSON.parse(<string>localStorage.getItem('showFullOperatorWindow'));
		this.filialRuleInFullOperatorWindow = JSON.parse(<string>localStorage.getItem('filialRuleInFullOperatorWindow'));
	}

	/**
	 * Life Cicle
	 */

	public async onCreatedOperadorCard() {
		Utilities.validateNovocodigo();

		this.promiseOperadorInfo = Http.get('/operador-info').then(({ data }: any) => {
			this.setOperadorInfo(data);
			return data;
		});

		this.promiseOrgoper = Http.get('/operador/organizacao').then((response) => response.data.data);
	}

	/**
	 * Instances components
	 */

	public getInstanceOperadorGrid(): TekGrid {
		return Metadata.getInstance<TekGrid>('operadorGrid');
	}

	/** **********************************
	 ****** Tab click and on enter *******
	 *********************************** */

	public async changeOperadorDetailTabs() {
		await PmLibUtil.timeout(200);
		const tabs = Metadata.getInstance<Tabs>('operadorDetailTabs');
		const { activeTab } = tabs;

		switch (activeTab) {
			case 0:
				this.clickOnTabOperadorFormEdit();
				break;
			case 1:
				this.clickOnTabOperadorPerfilAcesso();
				break;
			case 2:
				this.clickOnTabUnidadesRelacionadas();
				break;
			case 3:
				this.clickOnTabOrganizacoesRelacionadas();
				break;
			case 4:
				this.clickOnTabGrupoDeOperadores();
				break;
			default:
				break;
		}
	}

	private clickOnTabOperadorFormEdit() {
		const operadorFormEdit = Metadata.getInstance<PmLibFormSave>('operadorFormEdit');
		const idopermasterField = operadorFormEdit.getChildInstance<Select>('IDOPERMASTER');
		idopermasterField.readonly = this.operador.IDOPERMASTER !== 'S';
	}

	private clickOnTabOperadorPerfilAcesso() {
		const grid = Metadata.getInstance<TekGrid>('operadorPerfilDeAcessoGrid');
		grid.datasource.setFilter({ NRPARCNEGOCIO: this.currentOperador.NRPARCNEGOCIO, IDATIVO: 'S' });
	}

	private async clickOnTabUnidadesRelacionadas() {
		const grid = Metadata.getInstance<TekGrid>('unidadesRelacionadasGrid');
		if (grid.datasource instanceof TekRestDatasource) {
			grid.datasource.dynamicFilter = {
				IDATIVO: [{
					operation: 'EQUALS',
					relation: 'AND',
					value: 'S',
				}],
			};
		}

		try {
			await grid.datasource.addFilter('CDOPERADOR', this.currentOperador.CDOPERADOR);
		} catch (e) {
			PmLibDialogService.showErrorBackend(e);
		}
	}

	private async clickOnTabOrganizacoesRelacionadas() {
		const grid = Metadata.getInstance<TekGrid>('organizacoesRelacionadasGrid');
		grid.datasource.setFilter({ CDOPERADOR: this.currentOperador.CDOPERADOR });
	}

	private async clickOnTabGrupoDeOperadores() {
		const grid = Metadata.getInstance<TekGrid>('grupoDeOperadoresGrid');
		grid.datasource.setFilter({ CDOPERADOR: this.currentOperador.CDOPERADOR });
	}
}
